function run_tests(num_tests)

    LEN_MAX  =  7;
    VAL_MIN  = -9;
    VAL_MAX  =  9;

    P_SINGLE =  0.5;
    P_SCALAR =  0.3;
    P_NAN    =  0.15;
    P_REAL   =  0.5;
    P_ZERO   =  0.3;

    gen_real    = @(m, n) randi([VAL_MIN VAL_MAX], m, n) + ...
                                ifelse(rand(m, n) < P_NAN,
                                       ifelse(rand(m, n) < 0.5, NaN, NA),
                                       0);

    gen_complex = @(m, n) gen_real(m, n) + I * gen_real(m, n);

    check       = @(A, B) isequaln(A, B) && isequaln(imag(A), imag(B)) && isequal(isna(A), isna(B));

    minmax    = {@min, @max};
    cumminmax = {@cummin, @cummax};
    nanflags  = {'omitnan', 'includenan'};

    for t = 1 : num_tests
        printf('Test %d: ', t);
        fflush(stdout);

        % generate input matrices

        m = randi([0 LEN_MAX]);
        n = randi([0 LEN_MAX]);

        real = rand() < P_REAL;
        float = rand() < P_SINGLE;

        scalar = [false false];
        if (rand() < P_SCALAR)
            scalar(randi([1 2])) = true;
        end;

        F = {};
        S = {};
        for i = 1 : 2
            if scalar(i)
                s = ifelse(real, gen_real(1, 1), gen_complex(1, 1));
                F{i} = S{i} = s;
            else
                temp = ifelse(real,
                              gen_real(m, n),
                              gen_complex(m, n));

                temp = ifelse(rand(m, n) < P_ZERO,
                              0,
                              temp);

                F{i} = temp;
                if float
                    F{i} = single(F{i});
                end;
                S{i} = sparse(temp);
            end;
        end;

        % min/max: one array
        err_msg = 'Check %d failed.\nFunction: %s; dim: %d; flag: "%s".\n';

        for i = 1 : length(minmax)
            func = minmax{i};
            for dim = 1 : 2
                for j = 1 : length(nanflags)
                    nanflag = nanflags{j};

                    r1 = func(F{1}, [], dim, nanflag);
                    r2 = func(S{1}, [], dim, nanflag);

                    if !check(r1, full(r2))
                        fprintf(stderr, err_msg, 1, func2str(func), dim, nanflag); fdisp(stderr, F{1}); return;
                    end;

                    [r1 ix1] = func(F{1}, [], dim, nanflag);
                    [r2 ix2] = func(S{1}, [], dim, nanflag);

                    if !(check(r1, full(r2)) && isequal(ix1, ix2))
                        fprintf(stderr, err_msg, 2, func2str(func), dim, nanflag); fdisp(stderr, F{1}); return;
                    end;
                end;
            end;
        end;

        % min/max: two arrays
        err_msg = 'Check %d failed.\nFunction: %s; flag: "%s".\n';

        for i = 1 : length(minmax)
            func = minmax{i};
            for j = 1 : length(nanflags)
                nanflag = nanflags{j};

                r1 = func(F{1}, F{2}, nanflag);
                r2 = func(S{1}, S{2}, nanflag);

                if !check(r1, full(r2))
                    fprintf(stderr, err_msg, 3, func2str(func), nanflag); fdisp(stderr, F{1}); fdisp(stderr, F{2}); return;
                end;
            end;
        end;

        % cummin/cummax (one array)
        err_msg = 'Check %d failed.\nFunction: %s; dim: %d; flag: "%s".\n';

        for i = 1 : length(cumminmax)
            func = cumminmax{i};
            for dim = 1 : 2
                for j = 1 : length(nanflags)
                    nanflag = nanflags{j};

                    r1 = func(F{1}, dim, nanflag);
                    r2 = func(S{1}, dim, nanflag);

                    if !check(r1, full(r2))
                        fprintf(stderr, err_msg, 4, func2str(func), dim, nanflag); fdisp(stderr, F{1}); return;
                    end;

                    [r1 ix1] = func(F{1}, dim, nanflag);
                    [r2 ix2] = func(S{1}, dim, nanflag);

                    if !(check(r1, full(r2)) && isequal(ix1, ix2))
                        fprintf(stderr, err_msg, 5, func2str(func), dim, nanflag); fdisp(stderr, F{1}); return;
                    end;
                end;
            end;
        end;

        disp('passed');
    end;
